/*
	$Id: commit.c,v 1.7 2002/09/08 18:48:44 syl Exp $

	Copyright (C) 1999, 2000 Syllabub
	Maroontress Fast Software.
*/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xproto.h>
#include <X11/Xresource.h>
#include <stdlib.h>

#include <stdio.h>

#include "Resource.h"
#include "Text.h"

#include "phrase.h"
#include "connection.h"
#include "xim.h"
#include "message.h"
#include "status.h"
#include "predict.h"
#include "commit.h"

static void
RegisterPhrase(Display *disp, wchar_t *phonetic, wchar_t *ideograph)
{
    XICCEncodingStyle style = XCompoundTextStyle;
    XTextProperty t;
    Window root = DefaultRootWindow(disp);
    Atom atom = XInternAtom(disp, "XIM_JA_STUDY", False);
    wchar_t *wcs_list[2];

    wcs_list[0] = phonetic;
    wcs_list[1] = ideograph;
    if (XwcTextListToTextProperty(disp, wcs_list, 2, style, &t) < 0) {
	return;
    }
    XSetTextProperty(disp, root, &t, atom);
    XFree(t.value);
}

static SylText *
PhoneticText(Phrase *phr)
{
    Candidate *p;

    if ((p = LookupCandidateOfPhrase(phr)) == NULL)
	return NULL;

    return (p->phonetic == NULL) ? p->applicant : p->phonetic;
}

void
ClearString(Connection *con, InputContext *ic)
{
    ic->preedit_draw->clear(con, ic);
    ClearPrediction(con, ic);
}

void
CommitString(Connection *con, InputContext *ic)
{ 
    XIM_COMMIT_CHARS xim_commit;
    SerializedPacket *req;
    int len;
    XTextProperty t;
    wchar_t *wcs, *phonetic;

    if (LengthOfSylText(ic->preedit) <= 0)
	return;

    FreeSylText(ic->fixed);
    ic->fixed = ic->preedit;
    ic->preedit = CreateSylTextFromMBString("", False);

    if (ic->head != NULL) {
	if (ic->head->predicted != PHRASE_NOT_PREDICTED) {
	    phonetic = CreateWCStringFromSylText0(PhoneticText(ic->view));
	    wcs = CreateWCStringFromSylText0(ic->view->chosen);
	    RecordCommittedPhrase(phonetic, wcs);
	    free(phonetic); /* CreateWCStringFromSylText0() */
	}
	else {
	    phonetic = CreateWCStringFromSylText0(ic->fixed);
	    wcs = CreateWCStringFromPhrase(ic->head);
	    RegisterPhrase(con->disp, phonetic, wcs);
	    StudyPhrase(ic->head, ic->fixed);
	    RecordCommittedPhrase(phonetic, wcs);
	    free(phonetic); /* CreateWCStringFromSylText0() */
	}
	FreePhrase(ic->head);
	if (ic->candidate_window != None)
	    XUnmapWindow(con->disp, ic->candidate_window);
	ic->head = NULL;
	ic->view = NULL;
    }
    else {
	wcs = CreateWCStringFromSylText0(ic->fixed);
	RecordCommittedPhrase(wcs, wcs);
    }

    XwcTextListToTextProperty(con->disp, &wcs, 1, XCompoundTextStyle, &t);
    xim_commit.im_id = con->im_id;
    xim_commit.ic_id = ic->ic_id;
    xim_commit.flag = 2; /* XLookupChars */
    xim_commit.string.len = t.nitems;
    xim_commit.string.val = t.value;
    Send_XIM_COMMIT_CHARS(&xim_commit, &req, &len);
    SendIntoWire(con, OPCODE_XIM_COMMIT, req, len);
    Free_SerializedPacket(req);
    free(t.value); /* XwcTextListToTextProperty() */
    free(wcs); /* CreateWCStringFromSylText0() */
    ic->caret = 0;
}

void
EchoXIMForwardEvent(Connection *con, XIM_FORWARD_EVENT *ev)
{
    SerializedPacket *req;
    int len;

    ev->flag = 0;
    Send_XIM_FORWARD_EVENT(ev, &req, &len);
    SendIntoWire(con, OPCODE_XIM_FORWARD_EVENT, req, len);
    Free_SerializedPacket(req);
}
